# WARNING
#
# THIS IS AN AUTOGENERATED FILE.
#
# MANUAL CHANGES CAN AND WILL BE OVERWRITTEN.

import re

from setuptools import setup

PACKAGE_NAME = 'crypto_seller'

PACKAGE_DESCRIPTION = """
# Automatic Crypto Seller.

Automatic one-way trades.

This is a wrapper about [`fin_depo`](https://gitfub.space/Jmaa/fin-depo/) for
automatically trading one way trades, by using repeated trades
over a period.

The primary motivation is for trading low-volume crypto assets slowly without
affecting the price too much.

Supported sites:

- [KuCoin](https://www.kucoin.com/): Online crypto-currency exchange.

## Installation

Install dependencies:

```shell
pip install -r requirements.txt
```

## Usage

Run using from the top directory:

```shell
python -m crypto_seller
```

The script will now automatically sell assets over time. The interval between
sell-offs are randomized to avoid front-running, and the amounts are also
randomized to avoid too consistent behaviour.

The log will both be displayed in the shell, and be placed in a log file
`output/log.txt`. Every sell-off will be logged to the `output/trades.csv`
file, with as much information as possible. Keep both of these for tax
purposes, if relevant.

## Auditing information

The most relevant libraries for auditing are:

- [`fin_depo`](https://gitfub.space/Jmaa/fin-depo): Library for programmatic
  fetching of depository assets and in some cases allows for order placement.
  This is the library that reads balances and places market orders.
- [`fin_defs`](https://gitfub.space/Jmaa/fin-defs): Definitions of financial
  assets and instruments. Used by `fin_depo`.
- [`python-kucoin`](https://python-kucoin.readthedocs.io/en/latest/) is used by
  `fin_depo` for providing KuCoin support.

## Taxation

Some parts of the world suffers from weird and draconian taxation rules on
cryptocurrencies, so it helps to keep track of the relevant trading
information.

As mentioned above, keep (and backup) both the log file (`output/log.txt`) and
the trades file (`output/trades.csv`).

To help with tax reporting, it might be useful to sign up to tax oriented
websites. For example, [CryptoSkat](https://cryptoskat.dk/) seems to be the
most mature on the danish market, and does support KuCoin.

## TODO

- [ ] Parse configuration from json.
- [ ] Ensure that a failure during selling results in a safe winding down of the system.
      * Catch runtime errors when selling
      * Show errors to log.
      * Stop loop and exit with results, and error indicator.
- [ ] Document configuration
- [ ] Document code auditing
- [X] Ensure sell time is included in order details
- [X] Log all trades to CSV file.
- [X] Collect information during the run and output after run



# License

```
MIT License

Copyright (c) 2024 Jon Michael Aanes

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
```""".strip()

PACKAGE_DESCRIPTION_SHORT = """
Automatic one-way trades.""".strip()


def parse_version_file(text: str) -> str:
    match = re.match(r'^__version__\s*=\s*(["\'])([\d\.]+)\1$', text)
    if match is None:
        msg = 'Malformed _version.py file!'
        raise Exception(msg)
    return match.group(2)


with open(PACKAGE_NAME + '/_version.py') as f:
    version = parse_version_file(f.read())


def parse_requirements(text: str) -> list[str]:
    return text.strip().split('\n')


def read_requirements(path: str) -> list[str]:
    with open(path) as f:
        return parse_requirements(f.read())


REQUIREMENTS_MAIN = """
fin_defs @ git+https://gitfub.space/Jmaa/fin-defs.git
fin_depo @ git+https://gitfub.space/Jmaa/fin-depo.git"""

REQUIREMENTS_TEST = """
"""


setup(
    name=PACKAGE_NAME,
    version=version,
    description=PACKAGE_DESCRIPTION_SHORT,
    long_description=PACKAGE_DESCRIPTION,
    long_description_content_type='text/markdown',
    author='Jon Michael Aanes',
    author_email='jonjmaa@gmail.com',
    url='https://gitfub.space/Jmaa/' + PACKAGE_NAME,
    packages=[PACKAGE_NAME],
    install_requires=parse_requirements(REQUIREMENTS_MAIN),
    extras_require={
        'test': parse_requirements(REQUIREMENTS_TEST),
    },
    python_requires='>=3.9',
)
